@@ -8,6 +8,8 @@ structure.
 See https://github.com/ladybug-tools/radiance-folder-structure#radiance-folder-structure
 
 """
+import json
+import warnings
 import os
 import re
 import shutil
@@ -320,6 +322,18 @@ class ModelFolder(_Folder):
         """
         return self._get_folder('VIEW', full)
 
+    def receiver_folder(self, full=False):
+        """Receiver folder path.
+
+        Args:
+            full: A boolean to weather the path should be a full path or a relative path
+                (default: False).
+
+        Returns:
+            Path to folder.
+        """
+        return self._get_folder('RECEIVER', full)
+
     def _validate_config(self):
         """Validate config dictionary."""
         for k, v in self.CFG_KEYS.items():
@@ -329,6 +343,17 @@ class ModelFolder(_Folder):
                 assert i in self._config[k], \
                     '{} option is missing from {} section.'.format(i, k)
 
+    @property
+    def has_aperture(self):
+        """Returns true if model has at least one aperture."""
+        # get aperture files
+        aperture_files = self.aperture_files()
+
+        # check if aperture files is empty
+        if aperture_files:
+            return True
+        return False
+
     @property
     def has_aperture_group(self):
         """Returns true if model has at least one aperture group."""
@@ -410,6 +435,32 @@ class ModelFolder(_Folder):
         )
         return self._match_files(modifier_files, geometry_files)
 
+    def aperture_group_files_black(self, exclude=None, rel_path=False):
+        """Return list of files for black aperture groups.
+
+        Args:
+            exclude: Identifier of aperture groups to exclude from the list of black
+                aperture groups files. This input can be either a single aperture group
+                identifier or a list of aperture group identifiers.
+            rel_path: Set rel_path to True for getting full path to files. By
+                default the path is relative to study folder root.
+        """
+        if exclude and not isinstance(exclude, (list, tuple)):
+            exclude = [exclude]
+        else:
+            exclude = []
+        
+        states = self.aperture_groups_states(full=True)
+        blk_files = []
+        for aperture_group, ap_states in states.items():
+            if aperture_group in exclude:
+                continue
+            blk_file = os.path.normpath(ap_states[0]['black'])
+            blk_file = os.path.join(self.aperture_group_folder(full=rel_path), blk_file)
+            blk_files.append(self._as_posix(blk_file))
+
+        return blk_files
+
     def scene_files(self, black_out=False, rel_path=True):
         """Return list of files for scene.
 
@@ -470,6 +521,24 @@ class ModelFolder(_Folder):
         )
         return view_info_files
 
+    def receiver_files(self, rel_path=True):
+        """Return list of receiver files."""
+        cfg = self._config['RECEIVER']
+        pattern = cfg['receiver_pattern']
+        receiver_files = self._find_files(
+            self.receiver_folder(full=True), pattern, rel_path
+        )
+        return receiver_files
+
+    def receiver_info_file(self, rel_path=True):
+        """Return the receiver information file."""
+        cfg = self._config['RECEIVER']
+        pattern = cfg['info_pattern']
+        receiver_info_file = self._find_files(
+            self.receiver_folder(full=True), pattern, rel_path
+        )
+        return receiver_info_file[0]
+
     def aperture_groups(self, interior=False, reload=False):
         """List of apertures groups.
 
@@ -504,7 +573,7 @@ class ModelFolder(_Folder):
                 apt_group_folder, self._config['APERTURE-GROUP']['states'])
         return parse_states(states_file)
 
-    def combined_receivers(self, folder='receivers', auto_mtx_path=True):
+    def combined_receivers(self, folder='receiver', auto_mtx_path=False):
         """Write combined receiver files to folder.
 
         This function writes a combined receiver file of the aperture groups for all
@@ -516,25 +585,36 @@ class ModelFolder(_Folder):
         unique view matrix file.
 
         Arg:
-            folder: A path of the target folder to write files to (default: 'receivers').
+            folder: A path of the target folder to write files to (default: 'receiver').
             auto_mtx_path: If set to True, then the path of the view matrices will be
                 specified automatically.
         Returns:
             A dictionary containing grid identifiers as keys and the receiver rad files
             as values.
         """
-        grids = self.grid_data_all()
+        grids = self.grid_data_all() or []
+
         apt_group_folder = self.aperture_group_folder(full=False)
 
         states = self.aperture_groups_states(full=True)
+        rec_folder = os.path.join(
+            self.model_folder(True), folder
+        )
+        if not os.path.isdir(rec_folder):
+            os.mkdir(rec_folder)
 
-        if not os.path.isdir(folder):
-            os.mkdir(folder)
-
-        receiver_dict = dict()
+        receivers_info = []
 
         # find the light_path for each grid
-        for grid in grids.values():
+        for grid in grids:
+            if not 'light_path' in grid:
+                # The light-path for this grid is not set
+                # This grid will be ignored for 3/5 phase studies
+                warnings.warn(
+                    '%s sensor grid has no light-path. It will not be included in three '
+                    'or five phase studies.' % grid['name']
+                )
+                continue
             light_path = grid['light_path']
             # remove the static windows
             # another way is to check for the ones which has a mtx file.
@@ -546,11 +626,102 @@ class ModelFolder(_Folder):
                 grid['identifier'],
                 apt_group_folder,
                 aperture_groups,
-                folder, add_output_header=auto_mtx_path)
+                rec_folder, add_output_header=auto_mtx_path
+            )
+            receivers_info.append(
+                {
+                    'identifier': grid['identifier'],
+                    'count': grid['count'],
+                    'path': receiver_file,
+                    'aperture_groups': aperture_groups
+                }
+            )
 
-            receiver_dict[grid['identifier']] = receiver_file
+        receivers_info_file = os.path.join(rec_folder, '_info.json')
 
-        return receiver_dict
+        with open(receivers_info_file, 'w') as outf:
+            outf.write(json.dumps(receivers_info))
+
+        return receivers_info
+
+    def octree_scene_mapping(self):
+        """List of rad files for each state for aperture groups without transmission
+        matrix. These files can be used to create the octree for each specific state."""
+
+        two_phase, three_phase, five_phase = [], [], []
+
+        # two phase
+        # static apertures
+        if self.has_aperture:
+            two_phase.append(
+                {
+                    'identifier': '__static_apertures__',
+                    'scene_files': self.scene_files() + self.aperture_files() + \
+                        self.aperture_group_files_black(),
+                    'scene_files_direct': self.scene_files(black_out=True) + \
+                        self.aperture_files() + \
+                        self.aperture_group_files_black()
+                }
+            )
+
+        states = self.aperture_groups_states(full=True)
+        # add scene files for each state. Static apertures and all other aperture groups 
+        # will be black
+        for aperture_group, ap_states in states.items():
+            for state in ap_states:
+                if not 'tmtx' in state:
+                    pattern = '%s$' % state['default'].replace('./', '')
+                    two_phase.append(
+                        {
+                            'identifier': state['identifier'],
+                            'scene_files': self.scene_files() + \
+                                self.aperture_files(black_out=True) + \
+                                self._find_files(self.aperture_group_folder(full=True), 
+                                    pattern) + \
+                                self.aperture_group_files_black(exclude=aperture_group),
+                            'scene_files_direct': self.scene_files(black_out=True) + \
+                                self.aperture_files(black_out=True) + \
+                                self._find_files(self.aperture_group_folder(full=True), 
+                                    pattern) + \
+                                self.aperture_group_files_black(exclude=aperture_group)
+                        }
+                    )
+                else:
+                    # five phase
+                    pattern = '%s$' % state['direct'].replace('./', '')
+                    five_phase.append(
+                        {
+                        'identifier': state['identifier'],
+                        'scene_files_direct': self.scene_files(black_out=True) + \
+                            self.aperture_files(black_out=True) + \
+                            self._find_files(self.aperture_group_folder(full=True),
+                                pattern) + \
+                            self.aperture_group_files_black(exclude=aperture_group)
+                        }
+                    )
+
+        # three phase
+        three_phase.append(
+            {
+            'identifier': '__three_phase__',
+            'scene_files': self.scene_files() + self.aperture_files(),
+            'scene_files_direct': self.scene_files(black_out=True) + \
+                self.aperture_files(black_out=True)
+            }
+        )
+
+        scene_mapping = {
+            'two_phase': two_phase,
+            'three_phase': three_phase,
+            'five_phase': five_phase
+        }
+
+        scene_mapping_file = os.path.join(self.folder, 'scene_mapping.json')
+
+        with open(scene_mapping_file, 'w') as outf:
+            outf.write(json.dumps(scene_mapping, indent=2))
+
+        return scene_mapping
 
     def dynamic_scene(self, indoor=False, reload=False):
         """List of dynamic non-aperture geometries.
@@ -604,19 +775,13 @@ class ModelFolder(_Folder):
         """
 
         grid_info = self.grid_info(is_model=info_from_model)
-        grid_folder = self.grid_folder(full=True)
-
-        grid_data_dict = {}
 
         if not grid_info:
             return None
         else:
             for grid in grid_info:
-                grid_id = grid.pop('full_id')
-                grid_data_dict[grid_id] = parse_grid_json(
-                    os.path.join(grid_folder, grid_id + '.json'))
-                grid_data_dict[grid_id].update(grid)
-            return grid_data_dict
+                grid.pop('full_id')
+            return grid_info
 
     def write(self, folder_type=0, cfg=None, overwrite=False):
         """Write an empty model folder.
